#include <cstdlib>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "EasyBMP.h"

#define HEAD_SIZE 5

int main(int argc, char **argv) {
	int width, height, xpos, ypos;
	int x, y, p;
	int bufsize;
	int lcd;
	unsigned char *output;
	
	if(argc < 4) {
		printf("Usage: %s <image> <x> <y>\n", argv[0]);
		return 1;
	}
	
	BMFH header = GetBMFH(argv[1]);
	if(header.bfType != 19778) {
		printf("Can't read input file\n");
		return 1;
	}
	
	BMP input;
	
	input.ReadFromFile(argv[1]);
	width = input.TellWidth();
	height = input.TellHeight();
	xpos = atoi(argv[2]);
	ypos = atoi(argv[3]);
	
	if(xpos > 131 || ypos > 131) {
		printf("Invalid top left coordinates (%d,%d; max 131,131)\n", xpos, ypos);
		return 1;
	}
	
	if(width > 255 || height > 255) {
		printf("Image is too big (%dx%d, max 255x255)\n", width, height);
		return 1;
	}
	
	bufsize =  HEAD_SIZE+width*height*3;
	
	output = (unsigned char*)malloc(bufsize);
	
	output[0] = 0x00;
	output[1] = (unsigned char)xpos;
	output[2] = (unsigned char)ypos;
	output[3] = (unsigned char)width;
	output[4] = (unsigned char)height;
	
	p = HEAD_SIZE;
	
	for(y=0; y<height; y++) {
		for(x=0; x<width; x++) {
			output[p++] = input(x,y)->Red;
			output[p++] = input(x,y)->Green;
			output[p++] = input(x,y)->Blue;
		}
	}
	
	FILE *outfile;
	
	if(argc > 4) {
		outfile = fopen(argv[4], "w");
		if(!outfile) {
			printf("Can't open output file for writing: %s\n", argv[4]);
			return 1;
		}
		
		fwrite(output, bufsize, 1, outfile);
		fclose(outfile);
		free(output);
		return 0;
	}
	
	lcd = open("/dev/lcd", O_WRONLY);
	if(!lcd) {
		printf("Can't open LCD device\n");
		return 1;
	}
	
	write(lcd, output, bufsize);
	close(lcd);
	
	
	
	free(output);
	
	//printf("Bitmap file is %dx%d\n", input.TellWidth(), input.TellHeight());
	
	return 0;
}
